  xdef INTR_SW_TSK
  xdef StartExecution
  xdef SaveSP
  
  
  ;Externally referenced symbols.
  xref currTask
  xref Tasks
  xref Stacks
  xref taskResolutionTable

; INTR_SW_TSK transfers execution to the next task ready to run.

INTR_SW_TSK:
    sei              ; Disable interrupts during task switch
    clrh
    ldx currTask
    aslx
    lda Stacks,x
    psha
    lda Stacks+1,x
    tax
    pulh
    txs
    pulh
    cli
    rti


; This is the Helium scheduler.


StartExecution:
    ldx Tasks
    clrh
    lda taskResolutionTable,x
    sta currTask
    asla
    tax
    lda Stacks,x
    psha
    lda Stacks+1,x
    tax
    pulh
    txs
    ais #1
    rti             ; Execute the task
    
; Saves the SP register in the Stacks[] array associated with currTask
SaveSP:
    tsx              ; preserve the value of the stack pointer in H:X before
    pshx             ; store the value of the stack pointer on the stack.
    pshh
    lda 2,sp         ; Increment the stored value by 3 to account for the local variables allocated
    add #3           ; in TickISR (1 byte) and the return address for the function call (2 bytes)
    sta 2,sp
    bcc SPIncremented
    lda 1,sp
    adc #0
    sta 1,sp         ; Store the incremented value back on the stack
SPIncremented:
    clrh
    ldx currTask     ; Save the incremented value in the array
    aslx             ; element Stacks[currTask].
    lda 1,sp
    sta Stacks,x
    lda 2,sp
    sta Stacks+1,x
    ais #2           ; Deallocate stack frame and return.
    rts





